<?php

class Kakao
{
	const OAUTH_URL		= "https://kauth.kakao.com/oauth/token";

	private $key				= "";
	private $oauth_url		= "";
	private $code				= "";
	private $redirect_uri		= "";
	private $access_token	= "";
	private $refresh_token	= "";
	private $token_type		= "";
	private $expires_in		= "";

	public function __construct($key,$code,$redirect_uri) {		
		$this->key					= $key;
		$this->code				= $code;
		$this->redirect_uri		= trim(urlencode($redirect_uri));
		$this->oauth();
	}

	private function oauth(){
		$options			= array( 
			CURLOPT_URL => self::OAUTH_URL,
			CURLOPT_POSTFIELDS => sprintf( 'grant_type=authorization_code&client_id=%s&redirect_uri=%s&code=%s', $this->key, $this->redirect_uri, $this->code)
		);
		$response		= json_decode($this->request($options));
		if(!empty($response->access_token)){
			$this->access_token		= $response->access_token;
			$this->refresh_token		= $response->refresh_token;
			$this->token_type		= $response->token_type;
			$this->expires_in			= $response->expires_in;
		}
	}

	public function refresh_token(){
		$options			= array( 
			CURLOPT_URL => self::OAUTH_URL,
			CURLOPT_POSTFIELDS => sprintf( 'grant_type=refresh_token&client_id=%s&refresh_token=%s', $this->key, $this->refresh_token)
		);
		$response		= json_decode($this->request($options));
		if(!empty($response->access_token)){
			$this->access_token		= $response->access_token;
			$this->refresh_token		= $response->refresh_token;
			$this->token_type		= $response->token_type;
			$this->expires_in			= $response->expires_in;
		}
	}

	public function get($url, $opt=array()){
		$options			= array( 
			CURLOPT_URL => $url, 
			CURLOPT_HTTPHEADER =>array("Authorization: ".$this->token_type." ".$this->access_token)
		);
		if(!empty($opt)){
			foreach($opt  as $key => $value){
				$options[$key]		= $value;
			}
		}
		$response		= json_decode($this->request($options));
		return $response;
	}

	public function request($opt=array()) {
		$options				= array( 
			CURLOPT_SSL_VERIFYPEER => false, 
			CURLOPT_SSLVERSION => 1, 
			CURLOPT_POST => true, 
			CURLOPT_POSTFIELDS => false, 
			CURLOPT_RETURNTRANSFER => true
		);		

		if(!empty($opt)){
			foreach($opt  as $key => $value){
				$options[$key]		= $value;
			}
		}

		$curl_handle			= curl_init(); 
		curl_setopt_array($curl_handle, $options); 
		$response	= curl_exec($curl_handle); 
		curl_close($curl_handle);
		return $response;
    }
}





?>